import pygame

import character
import defs

class World(object):
    def __init__(self, background, castle, player):
        self.background = background
        self.castle     = castle
        self.player     = player
        self.enemies    = [character.Archer((450, 150)), character.Archer((400, 150))]
        self.missiles   = []
        
        self.screen = pygame.display.get_surface()
    
    def add_missile(self, missile):
        self.missiles.append(missile)
    
    def draw(self, (dx, dy)=(0, 0)):
        self.screen.blit(self.background, (dx, dy))
        for e in self.enemies:
            e.draw(self.screen, (dx, dy))
        self.screen.blit(self.castle, (dx, dy))
        for m in self.missiles:
            m.draw(self.screen, (dx, dy))
        self.player.draw(self.screen, (dx, dy))
    
    def update(self, delay, keys, events):
        # Updating missile positions
        for m in self.missiles:
            m.update(delay, events, self)
        
        # Updating actor positions
        actors = self.enemies + [self.player]
        for a in actors:
            a.think(keys, events, self)
            a.update(delay, events, keys, self)
        
        # Verifying collisions
        for m in self.missiles:
            if m.dz < 0:
                if m.z in defs.background_z_range:
                    pass
            elif m.dz > 0:
                if m.z in defs.foreground_z_range:
                    if self.player.rect.collidepoint((m.x, m.y)):
                        m.delete_flag = True
                        if self.player.state == defs.state_catch:
                            self.player.state_timer = 0
                            self.player.weapon = m
                        else:
                            self.player.rect.centerx = 0
        
        deleted = [m for m in self.missiles if m.can_delete()]
        if deleted:
            print "Deleted {0}".format(deleted)
        self.missiles = [m for m in self.missiles if not m.can_delete()]
        self.enemies  = [e for e in self.enemies  if not e.can_delete()]
